
//  (C) Copyright Edward Diener 2011-2015
//  Use, modification and distribution are subject to the Boost Software License,
//  Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt).

#if !defined(BOOST_VMD_DETAIL_MATCH_SINGLE_IDENTIFIER_HPP)
#define BOOST_VMD_DETAIL_MATCH_SINGLE_IDENTIFIER_HPP

#include <boost/vmd/detail/match_identifier_common.hpp>

#define BOOST_VMD_DETAIL_MATCH_SINGLE_IDENTIFIER(id,key) \
    BOOST_VMD_DETAIL_MATCH_IDENTIFIER_OP_CMP_IDS(id,key) \
/**/

#endif /* BOOST_VMD_DETAIL_MATCH_SINGLE_IDENTIFIER_HPP */
