/*!
@file
Defines the `BOOST_HANA_ADAPT_STRUCT` macro.

Copyright Louis Dionne 2013-2022
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE.md or copy at http://boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_HANA_ADAPT_STRUCT_HPP
#define BOOST_HANA_ADAPT_STRUCT_HPP

#include <boost/hana/fwd/adapt_struct.hpp>

#include <boost/hana/detail/struct_macros.hpp>

#endif // !BOOST_HANA_ADAPT_STRUCT_HPP
