# /* **************************************************************************
#  *                                                                          *
#  *     (C) Copyright Paul Mensonides 2002.
#  *     Distributed under the Boost Software License, Version 1.0. (See
#  *     accompanying file LICENSE_1_0.txt or copy at
#  *     http://www.boost.org/LICENSE_1_0.txt)
#  *                                                                          *
#  ************************************************************************** */
#
# /* Revised by Edward Diener (2020) */
#
# /* See http://www.boost.org for most recent version. */
#
# ifndef BOOST_PREPROCESSOR_SEQ_SIZE_512_HPP
# define BOOST_PREPROCESSOR_SEQ_SIZE_512_HPP
#
# define BOOST_PP_SEQ_SIZE_257(_) BOOST_PP_SEQ_SIZE_258
# define BOOST_PP_SEQ_SIZE_258(_) BOOST_PP_SEQ_SIZE_259
# define BOOST_PP_SEQ_SIZE_259(_) BOOST_PP_SEQ_SIZE_260
# define BOOST_PP_SEQ_SIZE_260(_) BOOST_PP_SEQ_SIZE_261
# define BOOST_PP_SEQ_SIZE_261(_) BOOST_PP_SEQ_SIZE_262
# define BOOST_PP_SEQ_SIZE_262(_) BOOST_PP_SEQ_SIZE_263
# define BOOST_PP_SEQ_SIZE_263(_) BOOST_PP_SEQ_SIZE_264
# define BOOST_PP_SEQ_SIZE_264(_) BOOST_PP_SEQ_SIZE_265
# define BOOST_PP_SEQ_SIZE_265(_) BOOST_PP_SEQ_SIZE_266
# define BOOST_PP_SEQ_SIZE_266(_) BOOST_PP_SEQ_SIZE_267
# define BOOST_PP_SEQ_SIZE_267(_) BOOST_PP_SEQ_SIZE_268
# define BOOST_PP_SEQ_SIZE_268(_) BOOST_PP_SEQ_SIZE_269
# define BOOST_PP_SEQ_SIZE_269(_) BOOST_PP_SEQ_SIZE_270
# define BOOST_PP_SEQ_SIZE_270(_) BOOST_PP_SEQ_SIZE_271
# define BOOST_PP_SEQ_SIZE_271(_) BOOST_PP_SEQ_SIZE_272
# define BOOST_PP_SEQ_SIZE_272(_) BOOST_PP_SEQ_SIZE_273
# define BOOST_PP_SEQ_SIZE_273(_) BOOST_PP_SEQ_SIZE_274
# define BOOST_PP_SEQ_SIZE_274(_) BOOST_PP_SEQ_SIZE_275
# define BOOST_PP_SEQ_SIZE_275(_) BOOST_PP_SEQ_SIZE_276
# define BOOST_PP_SEQ_SIZE_276(_) BOOST_PP_SEQ_SIZE_277
# define BOOST_PP_SEQ_SIZE_277(_) BOOST_PP_SEQ_SIZE_278
# define BOOST_PP_SEQ_SIZE_278(_) BOOST_PP_SEQ_SIZE_279
# define BOOST_PP_SEQ_SIZE_279(_) BOOST_PP_SEQ_SIZE_280
# define BOOST_PP_SEQ_SIZE_280(_) BOOST_PP_SEQ_SIZE_281
# define BOOST_PP_SEQ_SIZE_281(_) BOOST_PP_SEQ_SIZE_282
# define BOOST_PP_SEQ_SIZE_282(_) BOOST_PP_SEQ_SIZE_283
# define BOOST_PP_SEQ_SIZE_283(_) BOOST_PP_SEQ_SIZE_284
# define BOOST_PP_SEQ_SIZE_284(_) BOOST_PP_SEQ_SIZE_285
# define BOOST_PP_SEQ_SIZE_285(_) BOOST_PP_SEQ_SIZE_286
# define BOOST_PP_SEQ_SIZE_286(_) BOOST_PP_SEQ_SIZE_287
# define BOOST_PP_SEQ_SIZE_287(_) BOOST_PP_SEQ_SIZE_288
# define BOOST_PP_SEQ_SIZE_288(_) BOOST_PP_SEQ_SIZE_289
# define BOOST_PP_SEQ_SIZE_289(_) BOOST_PP_SEQ_SIZE_290
# define BOOST_PP_SEQ_SIZE_290(_) BOOST_PP_SEQ_SIZE_291
# define BOOST_PP_SEQ_SIZE_291(_) BOOST_PP_SEQ_SIZE_292
# define BOOST_PP_SEQ_SIZE_292(_) BOOST_PP_SEQ_SIZE_293
# define BOOST_PP_SEQ_SIZE_293(_) BOOST_PP_SEQ_SIZE_294
# define BOOST_PP_SEQ_SIZE_294(_) BOOST_PP_SEQ_SIZE_295
# define BOOST_PP_SEQ_SIZE_295(_) BOOST_PP_SEQ_SIZE_296
# define BOOST_PP_SEQ_SIZE_296(_) BOOST_PP_SEQ_SIZE_297
# define BOOST_PP_SEQ_SIZE_297(_) BOOST_PP_SEQ_SIZE_298
# define BOOST_PP_SEQ_SIZE_298(_) BOOST_PP_SEQ_SIZE_299
# define BOOST_PP_SEQ_SIZE_299(_) BOOST_PP_SEQ_SIZE_300
# define BOOST_PP_SEQ_SIZE_300(_) BOOST_PP_SEQ_SIZE_301
# define BOOST_PP_SEQ_SIZE_301(_) BOOST_PP_SEQ_SIZE_302
# define BOOST_PP_SEQ_SIZE_302(_) BOOST_PP_SEQ_SIZE_303
# define BOOST_PP_SEQ_SIZE_303(_) BOOST_PP_SEQ_SIZE_304
# define BOOST_PP_SEQ_SIZE_304(_) BOOST_PP_SEQ_SIZE_305
# define BOOST_PP_SEQ_SIZE_305(_) BOOST_PP_SEQ_SIZE_306
# define BOOST_PP_SEQ_SIZE_306(_) BOOST_PP_SEQ_SIZE_307
# define BOOST_PP_SEQ_SIZE_307(_) BOOST_PP_SEQ_SIZE_308
# define BOOST_PP_SEQ_SIZE_308(_) BOOST_PP_SEQ_SIZE_309
# define BOOST_PP_SEQ_SIZE_309(_) BOOST_PP_SEQ_SIZE_310
# define BOOST_PP_SEQ_SIZE_310(_) BOOST_PP_SEQ_SIZE_311
# define BOOST_PP_SEQ_SIZE_311(_) BOOST_PP_SEQ_SIZE_312
# define BOOST_PP_SEQ_SIZE_312(_) BOOST_PP_SEQ_SIZE_313
# define BOOST_PP_SEQ_SIZE_313(_) BOOST_PP_SEQ_SIZE_314
# define BOOST_PP_SEQ_SIZE_314(_) BOOST_PP_SEQ_SIZE_315
# define BOOST_PP_SEQ_SIZE_315(_) BOOST_PP_SEQ_SIZE_316
# define BOOST_PP_SEQ_SIZE_316(_) BOOST_PP_SEQ_SIZE_317
# define BOOST_PP_SEQ_SIZE_317(_) BOOST_PP_SEQ_SIZE_318
# define BOOST_PP_SEQ_SIZE_318(_) BOOST_PP_SEQ_SIZE_319
# define BOOST_PP_SEQ_SIZE_319(_) BOOST_PP_SEQ_SIZE_320
# define BOOST_PP_SEQ_SIZE_320(_) BOOST_PP_SEQ_SIZE_321
# define BOOST_PP_SEQ_SIZE_321(_) BOOST_PP_SEQ_SIZE_322
# define BOOST_PP_SEQ_SIZE_322(_) BOOST_PP_SEQ_SIZE_323
# define BOOST_PP_SEQ_SIZE_323(_) BOOST_PP_SEQ_SIZE_324
# define BOOST_PP_SEQ_SIZE_324(_) BOOST_PP_SEQ_SIZE_325
# define BOOST_PP_SEQ_SIZE_325(_) BOOST_PP_SEQ_SIZE_326
# define BOOST_PP_SEQ_SIZE_326(_) BOOST_PP_SEQ_SIZE_327
# define BOOST_PP_SEQ_SIZE_327(_) BOOST_PP_SEQ_SIZE_328
# define BOOST_PP_SEQ_SIZE_328(_) BOOST_PP_SEQ_SIZE_329
# define BOOST_PP_SEQ_SIZE_329(_) BOOST_PP_SEQ_SIZE_330
# define BOOST_PP_SEQ_SIZE_330(_) BOOST_PP_SEQ_SIZE_331
# define BOOST_PP_SEQ_SIZE_331(_) BOOST_PP_SEQ_SIZE_332
# define BOOST_PP_SEQ_SIZE_332(_) BOOST_PP_SEQ_SIZE_333
# define BOOST_PP_SEQ_SIZE_333(_) BOOST_PP_SEQ_SIZE_334
# define BOOST_PP_SEQ_SIZE_334(_) BOOST_PP_SEQ_SIZE_335
# define BOOST_PP_SEQ_SIZE_335(_) BOOST_PP_SEQ_SIZE_336
# define BOOST_PP_SEQ_SIZE_336(_) BOOST_PP_SEQ_SIZE_337
# define BOOST_PP_SEQ_SIZE_337(_) BOOST_PP_SEQ_SIZE_338
# define BOOST_PP_SEQ_SIZE_338(_) BOOST_PP_SEQ_SIZE_339
# define BOOST_PP_SEQ_SIZE_339(_) BOOST_PP_SEQ_SIZE_340
# define BOOST_PP_SEQ_SIZE_340(_) BOOST_PP_SEQ_SIZE_341
# define BOOST_PP_SEQ_SIZE_341(_) BOOST_PP_SEQ_SIZE_342
# define BOOST_PP_SEQ_SIZE_342(_) BOOST_PP_SEQ_SIZE_343
# define BOOST_PP_SEQ_SIZE_343(_) BOOST_PP_SEQ_SIZE_344
# define BOOST_PP_SEQ_SIZE_344(_) BOOST_PP_SEQ_SIZE_345
# define BOOST_PP_SEQ_SIZE_345(_) BOOST_PP_SEQ_SIZE_346
# define BOOST_PP_SEQ_SIZE_346(_) BOOST_PP_SEQ_SIZE_347
# define BOOST_PP_SEQ_SIZE_347(_) BOOST_PP_SEQ_SIZE_348
# define BOOST_PP_SEQ_SIZE_348(_) BOOST_PP_SEQ_SIZE_349
# define BOOST_PP_SEQ_SIZE_349(_) BOOST_PP_SEQ_SIZE_350
# define BOOST_PP_SEQ_SIZE_350(_) BOOST_PP_SEQ_SIZE_351
# define BOOST_PP_SEQ_SIZE_351(_) BOOST_PP_SEQ_SIZE_352
# define BOOST_PP_SEQ_SIZE_352(_) BOOST_PP_SEQ_SIZE_353
# define BOOST_PP_SEQ_SIZE_353(_) BOOST_PP_SEQ_SIZE_354
# define BOOST_PP_SEQ_SIZE_354(_) BOOST_PP_SEQ_SIZE_355
# define BOOST_PP_SEQ_SIZE_355(_) BOOST_PP_SEQ_SIZE_356
# define BOOST_PP_SEQ_SIZE_356(_) BOOST_PP_SEQ_SIZE_357
# define BOOST_PP_SEQ_SIZE_357(_) BOOST_PP_SEQ_SIZE_358
# define BOOST_PP_SEQ_SIZE_358(_) BOOST_PP_SEQ_SIZE_359
# define BOOST_PP_SEQ_SIZE_359(_) BOOST_PP_SEQ_SIZE_360
# define BOOST_PP_SEQ_SIZE_360(_) BOOST_PP_SEQ_SIZE_361
# define BOOST_PP_SEQ_SIZE_361(_) BOOST_PP_SEQ_SIZE_362
# define BOOST_PP_SEQ_SIZE_362(_) BOOST_PP_SEQ_SIZE_363
# define BOOST_PP_SEQ_SIZE_363(_) BOOST_PP_SEQ_SIZE_364
# define BOOST_PP_SEQ_SIZE_364(_) BOOST_PP_SEQ_SIZE_365
# define BOOST_PP_SEQ_SIZE_365(_) BOOST_PP_SEQ_SIZE_366
# define BOOST_PP_SEQ_SIZE_366(_) BOOST_PP_SEQ_SIZE_367
# define BOOST_PP_SEQ_SIZE_367(_) BOOST_PP_SEQ_SIZE_368
# define BOOST_PP_SEQ_SIZE_368(_) BOOST_PP_SEQ_SIZE_369
# define BOOST_PP_SEQ_SIZE_369(_) BOOST_PP_SEQ_SIZE_370
# define BOOST_PP_SEQ_SIZE_370(_) BOOST_PP_SEQ_SIZE_371
# define BOOST_PP_SEQ_SIZE_371(_) BOOST_PP_SEQ_SIZE_372
# define BOOST_PP_SEQ_SIZE_372(_) BOOST_PP_SEQ_SIZE_373
# define BOOST_PP_SEQ_SIZE_373(_) BOOST_PP_SEQ_SIZE_374
# define BOOST_PP_SEQ_SIZE_374(_) BOOST_PP_SEQ_SIZE_375
# define BOOST_PP_SEQ_SIZE_375(_) BOOST_PP_SEQ_SIZE_376
# define BOOST_PP_SEQ_SIZE_376(_) BOOST_PP_SEQ_SIZE_377
# define BOOST_PP_SEQ_SIZE_377(_) BOOST_PP_SEQ_SIZE_378
# define BOOST_PP_SEQ_SIZE_378(_) BOOST_PP_SEQ_SIZE_379
# define BOOST_PP_SEQ_SIZE_379(_) BOOST_PP_SEQ_SIZE_380
# define BOOST_PP_SEQ_SIZE_380(_) BOOST_PP_SEQ_SIZE_381
# define BOOST_PP_SEQ_SIZE_381(_) BOOST_PP_SEQ_SIZE_382
# define BOOST_PP_SEQ_SIZE_382(_) BOOST_PP_SEQ_SIZE_383
# define BOOST_PP_SEQ_SIZE_383(_) BOOST_PP_SEQ_SIZE_384
# define BOOST_PP_SEQ_SIZE_384(_) BOOST_PP_SEQ_SIZE_385
# define BOOST_PP_SEQ_SIZE_385(_) BOOST_PP_SEQ_SIZE_386
# define BOOST_PP_SEQ_SIZE_386(_) BOOST_PP_SEQ_SIZE_387
# define BOOST_PP_SEQ_SIZE_387(_) BOOST_PP_SEQ_SIZE_388
# define BOOST_PP_SEQ_SIZE_388(_) BOOST_PP_SEQ_SIZE_389
# define BOOST_PP_SEQ_SIZE_389(_) BOOST_PP_SEQ_SIZE_390
# define BOOST_PP_SEQ_SIZE_390(_) BOOST_PP_SEQ_SIZE_391
# define BOOST_PP_SEQ_SIZE_391(_) BOOST_PP_SEQ_SIZE_392
# define BOOST_PP_SEQ_SIZE_392(_) BOOST_PP_SEQ_SIZE_393
# define BOOST_PP_SEQ_SIZE_393(_) BOOST_PP_SEQ_SIZE_394
# define BOOST_PP_SEQ_SIZE_394(_) BOOST_PP_SEQ_SIZE_395
# define BOOST_PP_SEQ_SIZE_395(_) BOOST_PP_SEQ_SIZE_396
# define BOOST_PP_SEQ_SIZE_396(_) BOOST_PP_SEQ_SIZE_397
# define BOOST_PP_SEQ_SIZE_397(_) BOOST_PP_SEQ_SIZE_398
# define BOOST_PP_SEQ_SIZE_398(_) BOOST_PP_SEQ_SIZE_399
# define BOOST_PP_SEQ_SIZE_399(_) BOOST_PP_SEQ_SIZE_400
# define BOOST_PP_SEQ_SIZE_400(_) BOOST_PP_SEQ_SIZE_401
# define BOOST_PP_SEQ_SIZE_401(_) BOOST_PP_SEQ_SIZE_402
# define BOOST_PP_SEQ_SIZE_402(_) BOOST_PP_SEQ_SIZE_403
# define BOOST_PP_SEQ_SIZE_403(_) BOOST_PP_SEQ_SIZE_404
# define BOOST_PP_SEQ_SIZE_404(_) BOOST_PP_SEQ_SIZE_405
# define BOOST_PP_SEQ_SIZE_405(_) BOOST_PP_SEQ_SIZE_406
# define BOOST_PP_SEQ_SIZE_406(_) BOOST_PP_SEQ_SIZE_407
# define BOOST_PP_SEQ_SIZE_407(_) BOOST_PP_SEQ_SIZE_408
# define BOOST_PP_SEQ_SIZE_408(_) BOOST_PP_SEQ_SIZE_409
# define BOOST_PP_SEQ_SIZE_409(_) BOOST_PP_SEQ_SIZE_410
# define BOOST_PP_SEQ_SIZE_410(_) BOOST_PP_SEQ_SIZE_411
# define BOOST_PP_SEQ_SIZE_411(_) BOOST_PP_SEQ_SIZE_412
# define BOOST_PP_SEQ_SIZE_412(_) BOOST_PP_SEQ_SIZE_413
# define BOOST_PP_SEQ_SIZE_413(_) BOOST_PP_SEQ_SIZE_414
# define BOOST_PP_SEQ_SIZE_414(_) BOOST_PP_SEQ_SIZE_415
# define BOOST_PP_SEQ_SIZE_415(_) BOOST_PP_SEQ_SIZE_416
# define BOOST_PP_SEQ_SIZE_416(_) BOOST_PP_SEQ_SIZE_417
# define BOOST_PP_SEQ_SIZE_417(_) BOOST_PP_SEQ_SIZE_418
# define BOOST_PP_SEQ_SIZE_418(_) BOOST_PP_SEQ_SIZE_419
# define BOOST_PP_SEQ_SIZE_419(_) BOOST_PP_SEQ_SIZE_420
# define BOOST_PP_SEQ_SIZE_420(_) BOOST_PP_SEQ_SIZE_421
# define BOOST_PP_SEQ_SIZE_421(_) BOOST_PP_SEQ_SIZE_422
# define BOOST_PP_SEQ_SIZE_422(_) BOOST_PP_SEQ_SIZE_423
# define BOOST_PP_SEQ_SIZE_423(_) BOOST_PP_SEQ_SIZE_424
# define BOOST_PP_SEQ_SIZE_424(_) BOOST_PP_SEQ_SIZE_425
# define BOOST_PP_SEQ_SIZE_425(_) BOOST_PP_SEQ_SIZE_426
# define BOOST_PP_SEQ_SIZE_426(_) BOOST_PP_SEQ_SIZE_427
# define BOOST_PP_SEQ_SIZE_427(_) BOOST_PP_SEQ_SIZE_428
# define BOOST_PP_SEQ_SIZE_428(_) BOOST_PP_SEQ_SIZE_429
# define BOOST_PP_SEQ_SIZE_429(_) BOOST_PP_SEQ_SIZE_430
# define BOOST_PP_SEQ_SIZE_430(_) BOOST_PP_SEQ_SIZE_431
# define BOOST_PP_SEQ_SIZE_431(_) BOOST_PP_SEQ_SIZE_432
# define BOOST_PP_SEQ_SIZE_432(_) BOOST_PP_SEQ_SIZE_433
# define BOOST_PP_SEQ_SIZE_433(_) BOOST_PP_SEQ_SIZE_434
# define BOOST_PP_SEQ_SIZE_434(_) BOOST_PP_SEQ_SIZE_435
# define BOOST_PP_SEQ_SIZE_435(_) BOOST_PP_SEQ_SIZE_436
# define BOOST_PP_SEQ_SIZE_436(_) BOOST_PP_SEQ_SIZE_437
# define BOOST_PP_SEQ_SIZE_437(_) BOOST_PP_SEQ_SIZE_438
# define BOOST_PP_SEQ_SIZE_438(_) BOOST_PP_SEQ_SIZE_439
# define BOOST_PP_SEQ_SIZE_439(_) BOOST_PP_SEQ_SIZE_440
# define BOOST_PP_SEQ_SIZE_440(_) BOOST_PP_SEQ_SIZE_441
# define BOOST_PP_SEQ_SIZE_441(_) BOOST_PP_SEQ_SIZE_442
# define BOOST_PP_SEQ_SIZE_442(_) BOOST_PP_SEQ_SIZE_443
# define BOOST_PP_SEQ_SIZE_443(_) BOOST_PP_SEQ_SIZE_444
# define BOOST_PP_SEQ_SIZE_444(_) BOOST_PP_SEQ_SIZE_445
# define BOOST_PP_SEQ_SIZE_445(_) BOOST_PP_SEQ_SIZE_446
# define BOOST_PP_SEQ_SIZE_446(_) BOOST_PP_SEQ_SIZE_447
# define BOOST_PP_SEQ_SIZE_447(_) BOOST_PP_SEQ_SIZE_448
# define BOOST_PP_SEQ_SIZE_448(_) BOOST_PP_SEQ_SIZE_449
# define BOOST_PP_SEQ_SIZE_449(_) BOOST_PP_SEQ_SIZE_450
# define BOOST_PP_SEQ_SIZE_450(_) BOOST_PP_SEQ_SIZE_451
# define BOOST_PP_SEQ_SIZE_451(_) BOOST_PP_SEQ_SIZE_452
# define BOOST_PP_SEQ_SIZE_452(_) BOOST_PP_SEQ_SIZE_453
# define BOOST_PP_SEQ_SIZE_453(_) BOOST_PP_SEQ_SIZE_454
# define BOOST_PP_SEQ_SIZE_454(_) BOOST_PP_SEQ_SIZE_455
# define BOOST_PP_SEQ_SIZE_455(_) BOOST_PP_SEQ_SIZE_456
# define BOOST_PP_SEQ_SIZE_456(_) BOOST_PP_SEQ_SIZE_457
# define BOOST_PP_SEQ_SIZE_457(_) BOOST_PP_SEQ_SIZE_458
# define BOOST_PP_SEQ_SIZE_458(_) BOOST_PP_SEQ_SIZE_459
# define BOOST_PP_SEQ_SIZE_459(_) BOOST_PP_SEQ_SIZE_460
# define BOOST_PP_SEQ_SIZE_460(_) BOOST_PP_SEQ_SIZE_461
# define BOOST_PP_SEQ_SIZE_461(_) BOOST_PP_SEQ_SIZE_462
# define BOOST_PP_SEQ_SIZE_462(_) BOOST_PP_SEQ_SIZE_463
# define BOOST_PP_SEQ_SIZE_463(_) BOOST_PP_SEQ_SIZE_464
# define BOOST_PP_SEQ_SIZE_464(_) BOOST_PP_SEQ_SIZE_465
# define BOOST_PP_SEQ_SIZE_465(_) BOOST_PP_SEQ_SIZE_466
# define BOOST_PP_SEQ_SIZE_466(_) BOOST_PP_SEQ_SIZE_467
# define BOOST_PP_SEQ_SIZE_467(_) BOOST_PP_SEQ_SIZE_468
# define BOOST_PP_SEQ_SIZE_468(_) BOOST_PP_SEQ_SIZE_469
# define BOOST_PP_SEQ_SIZE_469(_) BOOST_PP_SEQ_SIZE_470
# define BOOST_PP_SEQ_SIZE_470(_) BOOST_PP_SEQ_SIZE_471
# define BOOST_PP_SEQ_SIZE_471(_) BOOST_PP_SEQ_SIZE_472
# define BOOST_PP_SEQ_SIZE_472(_) BOOST_PP_SEQ_SIZE_473
# define BOOST_PP_SEQ_SIZE_473(_) BOOST_PP_SEQ_SIZE_474
# define BOOST_PP_SEQ_SIZE_474(_) BOOST_PP_SEQ_SIZE_475
# define BOOST_PP_SEQ_SIZE_475(_) BOOST_PP_SEQ_SIZE_476
# define BOOST_PP_SEQ_SIZE_476(_) BOOST_PP_SEQ_SIZE_477
# define BOOST_PP_SEQ_SIZE_477(_) BOOST_PP_SEQ_SIZE_478
# define BOOST_PP_SEQ_SIZE_478(_) BOOST_PP_SEQ_SIZE_479
# define BOOST_PP_SEQ_SIZE_479(_) BOOST_PP_SEQ_SIZE_480
# define BOOST_PP_SEQ_SIZE_480(_) BOOST_PP_SEQ_SIZE_481
# define BOOST_PP_SEQ_SIZE_481(_) BOOST_PP_SEQ_SIZE_482
# define BOOST_PP_SEQ_SIZE_482(_) BOOST_PP_SEQ_SIZE_483
# define BOOST_PP_SEQ_SIZE_483(_) BOOST_PP_SEQ_SIZE_484
# define BOOST_PP_SEQ_SIZE_484(_) BOOST_PP_SEQ_SIZE_485
# define BOOST_PP_SEQ_SIZE_485(_) BOOST_PP_SEQ_SIZE_486
# define BOOST_PP_SEQ_SIZE_486(_) BOOST_PP_SEQ_SIZE_487
# define BOOST_PP_SEQ_SIZE_487(_) BOOST_PP_SEQ_SIZE_488
# define BOOST_PP_SEQ_SIZE_488(_) BOOST_PP_SEQ_SIZE_489
# define BOOST_PP_SEQ_SIZE_489(_) BOOST_PP_SEQ_SIZE_490
# define BOOST_PP_SEQ_SIZE_490(_) BOOST_PP_SEQ_SIZE_491
# define BOOST_PP_SEQ_SIZE_491(_) BOOST_PP_SEQ_SIZE_492
# define BOOST_PP_SEQ_SIZE_492(_) BOOST_PP_SEQ_SIZE_493
# define BOOST_PP_SEQ_SIZE_493(_) BOOST_PP_SEQ_SIZE_494
# define BOOST_PP_SEQ_SIZE_494(_) BOOST_PP_SEQ_SIZE_495
# define BOOST_PP_SEQ_SIZE_495(_) BOOST_PP_SEQ_SIZE_496
# define BOOST_PP_SEQ_SIZE_496(_) BOOST_PP_SEQ_SIZE_497
# define BOOST_PP_SEQ_SIZE_497(_) BOOST_PP_SEQ_SIZE_498
# define BOOST_PP_SEQ_SIZE_498(_) BOOST_PP_SEQ_SIZE_499
# define BOOST_PP_SEQ_SIZE_499(_) BOOST_PP_SEQ_SIZE_500
# define BOOST_PP_SEQ_SIZE_500(_) BOOST_PP_SEQ_SIZE_501
# define BOOST_PP_SEQ_SIZE_501(_) BOOST_PP_SEQ_SIZE_502
# define BOOST_PP_SEQ_SIZE_502(_) BOOST_PP_SEQ_SIZE_503
# define BOOST_PP_SEQ_SIZE_503(_) BOOST_PP_SEQ_SIZE_504
# define BOOST_PP_SEQ_SIZE_504(_) BOOST_PP_SEQ_SIZE_505
# define BOOST_PP_SEQ_SIZE_505(_) BOOST_PP_SEQ_SIZE_506
# define BOOST_PP_SEQ_SIZE_506(_) BOOST_PP_SEQ_SIZE_507
# define BOOST_PP_SEQ_SIZE_507(_) BOOST_PP_SEQ_SIZE_508
# define BOOST_PP_SEQ_SIZE_508(_) BOOST_PP_SEQ_SIZE_509
# define BOOST_PP_SEQ_SIZE_509(_) BOOST_PP_SEQ_SIZE_510
# define BOOST_PP_SEQ_SIZE_510(_) BOOST_PP_SEQ_SIZE_511
# define BOOST_PP_SEQ_SIZE_511(_) BOOST_PP_SEQ_SIZE_512
# define BOOST_PP_SEQ_SIZE_512(_) BOOST_PP_SEQ_SIZE_513
#
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_258 258
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_259 259
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_260 260
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_261 261
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_262 262
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_263 263
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_264 264
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_265 265
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_266 266
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_267 267
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_268 268
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_269 269
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_270 270
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_271 271
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_272 272
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_273 273
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_274 274
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_275 275
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_276 276
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_277 277
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_278 278
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_279 279
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_280 280
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_281 281
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_282 282
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_283 283
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_284 284
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_285 285
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_286 286
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_287 287
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_288 288
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_289 289
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_290 290
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_291 291
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_292 292
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_293 293
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_294 294
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_295 295
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_296 296
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_297 297
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_298 298
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_299 299
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_300 300
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_301 301
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_302 302
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_303 303
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_304 304
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_305 305
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_306 306
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_307 307
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_308 308
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_309 309
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_310 310
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_311 311
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_312 312
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_313 313
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_314 314
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_315 315
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_316 316
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_317 317
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_318 318
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_319 319
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_320 320
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_321 321
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_322 322
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_323 323
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_324 324
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_325 325
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_326 326
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_327 327
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_328 328
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_329 329
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_330 330
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_331 331
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_332 332
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_333 333
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_334 334
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_335 335
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_336 336
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_337 337
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_338 338
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_339 339
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_340 340
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_341 341
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_342 342
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_343 343
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_344 344
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_345 345
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_346 346
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_347 347
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_348 348
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_349 349
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_350 350
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_351 351
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_352 352
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_353 353
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_354 354
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_355 355
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_356 356
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_357 357
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_358 358
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_359 359
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_360 360
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_361 361
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_362 362
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_363 363
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_364 364
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_365 365
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_366 366
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_367 367
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_368 368
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_369 369
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_370 370
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_371 371
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_372 372
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_373 373
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_374 374
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_375 375
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_376 376
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_377 377
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_378 378
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_379 379
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_380 380
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_381 381
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_382 382
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_383 383
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_384 384
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_385 385
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_386 386
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_387 387
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_388 388
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_389 389
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_390 390
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_391 391
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_392 392
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_393 393
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_394 394
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_395 395
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_396 396
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_397 397
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_398 398
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_399 399
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_400 400
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_401 401
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_402 402
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_403 403
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_404 404
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_405 405
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_406 406
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_407 407
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_408 408
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_409 409
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_410 410
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_411 411
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_412 412
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_413 413
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_414 414
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_415 415
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_416 416
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_417 417
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_418 418
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_419 419
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_420 420
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_421 421
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_422 422
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_423 423
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_424 424
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_425 425
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_426 426
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_427 427
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_428 428
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_429 429
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_430 430
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_431 431
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_432 432
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_433 433
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_434 434
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_435 435
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_436 436
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_437 437
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_438 438
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_439 439
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_440 440
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_441 441
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_442 442
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_443 443
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_444 444
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_445 445
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_446 446
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_447 447
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_448 448
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_449 449
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_450 450
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_451 451
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_452 452
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_453 453
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_454 454
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_455 455
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_456 456
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_457 457
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_458 458
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_459 459
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_460 460
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_461 461
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_462 462
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_463 463
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_464 464
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_465 465
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_466 466
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_467 467
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_468 468
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_469 469
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_470 470
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_471 471
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_472 472
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_473 473
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_474 474
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_475 475
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_476 476
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_477 477
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_478 478
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_479 479
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_480 480
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_481 481
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_482 482
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_483 483
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_484 484
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_485 485
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_486 486
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_487 487
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_488 488
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_489 489
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_490 490
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_491 491
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_492 492
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_493 493
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_494 494
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_495 495
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_496 496
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_497 497
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_498 498
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_499 499
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_500 500
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_501 501
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_502 502
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_503 503
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_504 504
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_505 505
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_506 506
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_507 507
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_508 508
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_509 509
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_510 510
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_511 511
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_512 512
# define BOOST_PP_SEQ_SIZE_BOOST_PP_SEQ_SIZE_513 513
#
# endif
